#!/usr/bin/python3

# Sample python plugin
# For more information see https://collectd.org/documentation/manpages/collectd-python.5.shtml

import sys
import time
#from math import pi, sin
import os
import socket
import threading
import collectd
import logging
import json
import subprocess
import requests
from systemd import journal
import time
import sysrepo as sr

collectd_path = '/tmp/collectd_sock'
PLUGIN_NAME = 'k8s_notification'
#tm = time.time()

class pod_status_entry(object):
    pod_name = ""
    pod_status = ""
    pod_namespace = ""
    timestamp = time.time()
    def __init__(self, name, status, namespace, timestamp):
        self.pod_name = name
        self.pod_status = status
        self.pod_namespace = namespace
        self.timestamp = timestamp

class status_entry(object):
    name = ""
    namespace = ""
    timestamp = time.time()
    def __init__(self, name_, namespace_, timestamp_):
        self.name = name_
        self.namespace = namespace_
        self.timestamp = timestamp_

class rs_status_entry(object):
    name = ""
    namespace = ""
    replicas = 0
    timestamp = time.time()
    def __init__(self, name_, namespace_, replicas_, timestamp_):
        self.name = name_
        self.namespace = namespace_
        self.timestamp = timestamp_
        self.replicas = replicas_

class pv_status_entry(object):
    name = ""
    status = ""
    size = ""
    timestamp = time.time()
    def __init__(self, name_, size_, status_, timestamp_):
        self.name = name_
        self.status = status_
        self.size = size_
        self.timestamp = timestamp_

class PodStatus(object):
    def __init__(self):
        collectd.info("plugin load: k8s_notification successfully loaded.")
        self.PodStatusDB = list()
        self.VmStatusDB = list()
        self.SvcDB = list()
        self.ReplicasetDB = list()
        self.StatefulsetDB = list()
        self.DeploymentDB = list()
        self.PvDB = list()
        
    def print_pod_stats_db(self, time):
        log("------  pod status list, number of entries: " + str(len(self.PodStatusDB)) + "  " + str(time) + "  -------")
        for entry in range(len(self.PodStatusDB)):
            collectd.info("%s: %s:%s %s %s" % (PLUGIN_NAME, self.PodStatusDB[entry].pod_name, self.PodStatusDB[entry].pod_namespace, self.PodStatusDB[entry].pod_status, self.PodStatusDB[entry].timestamp))

        log("------  vm status list, number of entries: " + str(len(self.VmStatusDB)) + "  " + str(time) + "  -------")
        for entry in range(len(self.VmStatusDB)):
            collectd.info("%s: %s:%s %s %s" % (PLUGIN_NAME, self.VmStatusDB[entry].pod_name, self.VmStatusDB[entry].pod_namespace, self.VmStatusDB[entry].pod_status, self.PodStatusDB[entry].timestamp))

        log("------  Service list, number of entries: " + str(len(self.SvcDB)) + "  " + str(time) + "  -------")
        for entry in range(len(self.SvcDB)):
            collectd.info("%s: %s %s" % (PLUGIN_NAME, self.SvcDB[entry].name, self.SvcDB[entry].timestamp))

        log("------  Replicaset list, number of entries: " + str(len(self.ReplicasetDB)) + "  " + str(time) + "  -------")
        for entry in range(len(self.ReplicasetDB)):
            collectd.info("%s: %s %s %s" % (PLUGIN_NAME, self.ReplicasetDB[entry].name, self.ReplicasetDB[entry].replicas, self.ReplicasetDB[entry].timestamp))

        log("------  Statefulset list, number of entries: " + str(len(self.StatefulsetDB)) + "  " + str(time) + "  -------")
        for entry in range(len(self.StatefulsetDB)):
            collectd.info("%s: %s %s" % (PLUGIN_NAME, self.StatefulsetDB[entry].name, self.StatefulsetDB[entry].timestamp))

        log("------  Deployment list, number of entries: " + str(len(self.DeploymentDB)) + "  " + str(time) + "  -------")
        for entry in range(len(self.DeploymentDB)):
            collectd.info("%s: %s %s" % (PLUGIN_NAME, self.DeploymentDB[entry].name, self.DeploymentDB[entry].timestamp))

        log("------  PV list, number of entries: " + str(len(self.PvDB)) + "  " + str(time) + "  -------")
        for entry in range(len(self.PvDB)):
            collectd.info("%s: %s status: %s capacity: %s %s" % (PLUGIN_NAME, self.PvDB[entry].name, self.PvDB[entry].status, self.PvDB[entry].size, self.PvDB[entry].timestamp))

    def send_notification(self, pod_name, state, classifier, namespace=None):
        if namespace:
            message = 'Status of %s:%s changed to %s' % (pod_name, namespace, state)
        else:
            message = 'Status of %s changed to %s' % (pod_name, state)
        severity = 1
        if state == 'Running':
            severity = 4
        try:
            with open('/config/collectd-hostname', 'r') as f:
                host = f.readline().strip()
        except:
            host=socket.gethostname()
        notif = collectd.Notification(plugin=PLUGIN_NAME,
                                      plugin_instance=classifier,
                                      type="gauge",
                                      severity=severity,
                                      message=message,
                                      host=host)
        notif.dispatch()


    def send_pod_create_notification(self, pod_name, state, classifier, namespace=None):
        if namespace:
            message = 'Created pod %s in namespace %s; status %s' % (pod_name, namespace, state)
        else:
            message = 'Created pod %s, current status %s' % (pod_name, state)
        severity = 1
        if state == 'Running':
            severity = 4
        try:
            with open('/config/collectd-hostname', 'r') as f:
                host = f.readline().strip()
        except:
            host=socket.gethostname()
        notif = collectd.Notification(plugin=PLUGIN_NAME,
                                      plugin_instance=classifier,
                                      type="gauge",
                                      severity=severity,
                                      message=message,
                                      host=host)
        notif.dispatch()

    def send_vm_create_notification(self, vm_name, state, classifier, namespace=None):
        if namespace:
            message = 'Created vm %s in namespace %s; status %s' % (vm_name, namespace, state)
        else:
            message = 'Created vm %s, current status %s' % (vm_name, state)
        severity = 1
        if state == 'Running':
            severity = 4
        try:
            with open('/config/collectd-hostname', 'r') as f:
                host = f.readline().strip()
        except:
            host=socket.gethostname()
        notif = collectd.Notification(plugin=PLUGIN_NAME,
                                      plugin_instance=classifier,
                                      type="gauge",
                                      severity=severity,
                                      message=message,
                                      host=host)
        notif.dispatch()

    def send_pod_delete_notification(self, element, name, classifier, namespace=None):
        if namespace:
            message = '%s %s deleted in namespace %s' % (element, name, namespace)
        else:
            message = '%s %s deleted' % (element, name)

        severity = 4
        try:
            with open('/config/collectd-hostname', 'r') as f:
                host = f.readline().strip()
        except:
            host=socket.gethostname()
        notif = collectd.Notification(plugin=PLUGIN_NAME,
                                      plugin_instance=classifier,
                                      type="gauge",
                                      severity=severity,
                                      message=message,
                                      host=host)
        notif.dispatch()

    def send_create_notification(self, element, action, name, classifier, namespace=None):

        if namespace:
            message = '%s %s %s in namespace %s' % (element, name, action, namespace)
        else:
            message = '%s %s %s' % (element, name, action)

        severity = 4

        try:
            with open('/config/collectd-hostname', 'r') as f:
                host = f.readline().strip()
        except:
            host=socket.gethostname()
        notif = collectd.Notification(plugin=PLUGIN_NAME,
                                      plugin_instance=classifier,
                                      type="gauge",
                                      severity=severity,
                                      message=message,
                                      host=host)
        notif.dispatch()

    def send_pv_create_notification(self, name, action, classifier, size=None, status=None):

        message = '%s PV %s with capacity %s' % (action, name, size)
        severity = 4

        try:
            with open('/config/collectd-hostname', 'r') as f:
                host = f.readline().strip()
        except:
            host=socket.gethostname()
        notif = collectd.Notification(plugin=PLUGIN_NAME,
                                      plugin_instance=classifier,
                                      type="gauge",
                                      severity=severity,
                                      message=message,
                                      host=host)
        notif.dispatch()

    def send_pv_update_notification(self, name, classifier, size=None, status=None):
        if status == None:
            message = 'PV %s capacity changed to %s' % (name, size)
        else:
            message = 'PV %s status changed to %s' % (name, status)

        severity = 4

        try:
            with open('/config/collectd-hostname', 'r') as f:
                host = f.readline().strip()
        except:
            host=socket.gethostname()
        notif = collectd.Notification(plugin=PLUGIN_NAME,
                                      plugin_instance=classifier,
                                      type="gauge",
                                      severity=severity,
                                      message=message,
                                      host=host)
        notif.dispatch()


    def send_rs_pod_notification(self, classifier, operation, rs_name, pod_name):
        message = 'Replicaset %s: %s pod %s' % (rs_name, operation, pod_name)

        severity = 4
        try:
            with open('/config/collectd-hostname', 'r') as f:
                host = f.readline().strip()
        except:
            host=socket.gethostname()
        notif = collectd.Notification(plugin=PLUGIN_NAME,
                                      plugin_instance=classifier,
                                      type="gauge",
                                      severity=severity,
                                      message=message,
                                      host=host)
        notif.dispatch()

    def send_ss_pod_notification(self, classifier, operation, ss_name, pod_name):
        message = 'Statefulset %s: %s pod %s' % (ss_name, operation, pod_name)

        severity = 4
        try:
            with open('/config/collectd-hostname', 'r') as f:
                host = f.readline().strip()
        except:
            host=socket.gethostname()
        notif = collectd.Notification(plugin=PLUGIN_NAME,
                                      plugin_instance=classifier,
                                      type="gauge",
                                      severity=severity,
                                      message=message,
                                      host=host)
        notif.dispatch()


    def send_depl_scaling_notification(self, depl_name, operation, classifier, rs_name, replicas):
        message = 'Deployment %s scaled %s replicaset %s to %s' % (depl_name, operation, rs_name, replicas)

        severity = 4
        try:
            with open('/config/collectd-hostname', 'r') as f:
                host = f.readline().strip()
        except:
            host=socket.gethostname()
        notif = collectd.Notification(plugin=PLUGIN_NAME,
                                      plugin_instance=classifier,
                                      type="gauge",
                                      severity=severity,
                                      message=message,
                                      host=host)
        notif.dispatch()

    def read_function(self):
        collectd.info("k8s_notification plugin : read called")
        current_time = time.time()
        
        self.print_pod_stats_db(current_time)
        all_pods = ""
        all_vms = ""
        all_serices = ""
        all_statefulsets = ""
        all_replicasets = ""
        all_deployments = ""
        all_pv = ""

        try:
            cmd = "kubectl get deployment -A"
            all_deployments = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; %s" %(cmd), shell=True)).decode('utf8', errors='ignore')

            deployments = set()
            if all_deployments != "":
                for line in all_deployments.split('\n')[1:]:
                    if line != "":
                        deployment_name = line.split()[1]
                        #log(deployment_name)
                        deployment_namespace = line.split()[0]
                        deployments.add(deployment_name)

                        found = False
                        for i in range(len(self.DeploymentDB)):
                            if self.DeploymentDB[i].name == deployment_name:
                                found = True
                                break

                        if not found:
                            collectd.info("%s: Adding entry for deployment: %s " % (PLUGIN_NAME, deployment_name))
                            self.DeploymentDB.append(status_entry(deployment_name, deployment_namespace, current_time))
                            classifier="deployment:"+deployment_namespace+":"+deployment_name
                            self.send_create_notification("Deployment", "created", deployment_name, classifier, deployment_namespace)

            for entry in self.DeploymentDB:
                if entry.name not in deployments:
                    collectd.info("%s: %s %s" % (PLUGIN_NAME, "Removing deployment entry:", entry.name))
                    classifier="deployment:"+entry.namespace+":"+entry.name
                    self.send_create_notification("Deployment", "deleted", entry.name, classifier, entry.namespace)
                    self.DeploymentDB.remove(entry)

        except Exception as e:
            collectd.error("k8s_notification plugin: read function failed - {}".format(e))


        try:
            cmd = "kubectl get replicaset -A"
            all_replicasets = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; %s" %(cmd), shell=True)).decode('utf8', errors='ignore')

            replicasets = set()
            if all_replicasets != "":
                for line in all_replicasets.split('\n')[1:]:
                    if line != "":
                        replicaset_name = line.split()[1]
                        #log(replicaset_name)
                        replicaset_namespace = line.split()[0]
                        replicasets.add(replicaset_name)

                        found = False
                        for i in range(len(self.ReplicasetDB)):
                            if self.ReplicasetDB[i].name == replicaset_name:
                                found = True
                                break

                        if not found:
                            collectd.info("%s: Adding entry for replicaset: %s " % (PLUGIN_NAME, replicaset_name))
                            self.ReplicasetDB.append(rs_status_entry(replicaset_name, replicaset_namespace, 0, current_time))
                            classifier="replicaset:"+replicaset_namespace+":"+replicaset_name
                            self.send_create_notification("Replicaset", "created", replicaset_name, classifier, replicaset_namespace)

            for entry in self.ReplicasetDB:
                if entry.name not in replicasets:
                    collectd.info("%s: %s %s" % (PLUGIN_NAME, "Removing replicaset entry:", entry.name))
                    classifier="replicaset:"+entry.namespace+":"+entry.name
                    self.send_create_notification("Replicaset", "deleted", entry.name, classifier, entry.namespace)
                    self.ReplicasetDB.remove(entry)

        except Exception as e:
            collectd.error("k8s_notification plugin: read function failed - {}".format(e))


        try:
            cmd = "kubectl get statefulset -A"
            all_statefulsets = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; %s" %(cmd), shell=True)).decode('utf8', errors='ignore')

            statefulsets = set()
            if all_statefulsets != "":
                for line in all_statefulsets.split('\n')[1:]:
                    if line != "":
                        statefulset_name = line.split()[1]
                        #log(statefulset_name)
                        statefulset_namespace = line.split()[0]
                        statefulsets.add(statefulset_name)

                        found = False
                        for i in range(len(self.StatefulsetDB)):
                            if self.StatefulsetDB[i].name == statefulset_name:
                                found = True
                                break

                        if not found:
                            collectd.info("%s: Adding entry for statefulset: %s " % (PLUGIN_NAME, statefulset_name))
                            self.StatefulsetDB.append(status_entry(statefulset_name, statefulset_namespace, current_time))
                            classifier="statefulset:"+statefulset_namespace+":"+statefulset_name
                            self.send_create_notification("Statefulset", "created", statefulset_name, classifier, statefulset_namespace)

            for entry in self.StatefulsetDB:
                if entry.name not in statefulsets:
                    collectd.info("%s: %s %s" % (PLUGIN_NAME, "Removing statefulset entry:", entry.name))
                    classifier="statefulset:"+entry.namespace+":"+entry.name
                    self.send_create_notification("Statefulset", "deleted", entry.name, classifier, entry.namespace)
                    self.StatefulsetDB.remove(entry)

        except Exception as e:
            collectd.error("k8s_notification plugin: read function failed - {}".format(e))


        try:
            cmd = "kubectl get pods -A  | grep -v \"monitoring \""
            all_pods = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; %s" %(cmd), shell=True)).decode('utf8', errors='ignore')
            
            pods = set()
            if all_pods != "":
                for line in all_pods.split('\n')[1:]:
                    if line != "":
                        pod_name = line.split()[1]
                        #log(pod_name)
                        pod_status = line.split()[3]
                        pod_namespace = line.split()[0]
                        #log(pod_status)
                        pod_name_namespace=pod_name+":"+pod_namespace
                        print(pod_name_namespace)
                        pods.add(pod_name_namespace)
                        
                        
                        found = False
                        for i in range(len(self.PodStatusDB)):
                            if self.PodStatusDB[i].pod_name == pod_name and self.PodStatusDB[i].pod_namespace == pod_namespace:
                                found = True
                                break

                        classifier="pod:"+pod_namespace+":"+pod_name
                        if not found:
                            collectd.info("%s: Adding entry for pod: %s, namespace: %s, state: %s" % (PLUGIN_NAME, pod_name, pod_namespace, pod_status))
                            self.PodStatusDB.append(pod_status_entry(pod_name, pod_status, pod_namespace, current_time))
                            self.send_pod_create_notification(pod_name, pod_status, classifier, pod_namespace)

                            # send replicaset create pod notification
                            for rs in self.ReplicasetDB:
                                if rs.name in pod_name:
                                    rs_classifier="rs:"+rs.namespace+":"+rs.name
                                    self.send_rs_pod_notification(rs_classifier, "created", rs.name, pod_name)
                                    rs.replicas=rs.replicas+1
                                    # send deployment scaling replicaset notification
                                    for depl in self.DeploymentDB:
                                        if depl.name in rs.name:
                                            depl_classifier="deployment:"+depl.namespace+":"+depl.name
                                            self.send_depl_scaling_notification(depl.name, "up", depl_classifier, rs.name, rs.replicas)

                            # send statefulset create pod notification
                            for ss in self.StatefulsetDB:
                                if ss.name in pod_name:
                                    ss_classifier="statefulset:"+ss.namespace+":"+ss.name
                                    self.send_ss_pod_notification(ss_classifier, "created", ss.name, pod_name)

                        else:
                            if self.PodStatusDB[i].pod_status != pod_status:
                                collectd.info("%s: Sending notification for pod: %s %s -> %s" % (PLUGIN_NAME, pod_name, self.PodStatusDB[i].pod_status, pod_status))
                                self.send_notification(pod_name, pod_status, classifier, pod_namespace)
                                self.PodStatusDB[i].pod_status = pod_status

        except Exception as e:
            collectd.error("k8s_notification plugin: read function failed - {}".format(e))

        try:        
            cmd = "kubectl get vm -A | grep -v No\ resources\ found  | grep -v NAME"
            all_vms = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; %s" %(cmd), shell=True)).decode('utf8', errors='ignore')

            vms = set()
            if all_vms != "":
                for line in all_vms.split('\n'):
                    if line != "":
                        vm_name = line.split()[1]
                        #log(vm_name)
                        vm_status = line.split()[3]
                        #log(vm_status)
                        vm_namespace = line.split()[0]
                        vm_name_namespace=vm_name+":"+vm_namespace
                        vms.add(vm_name_namespace)

                        found = False
                        for i in range(len(self.VmStatusDB)):
                            if self.VmStatusDB[i].pod_name == vm_name and self.VmStatusDB[i].pod_namespace == vm_namespace:
                                found = True
                                break
                        classifier="vm:"+vm_namespace+":"+vm_name

                        if not found:
                            collectd.info("%s: Adding entry for vm: %s, namespace: %s, state: %s" % (PLUGIN_NAME, vm_name, vm_namespace, vm_status))
                            self.VmStatusDB.append(pod_status_entry(vm_name, vm_status, vm_namespace, current_time))
                            self.send_vm_create_notification(vm_name, vm_status, classifier, vm_namespace)
                        else:
                            if self.VmStatusDB[i].pod_status != vm_status:
                                collectd.info("%s: Sending notification for vm: %s %s -> %s" % (PLUGIN_NAME, vm_name, self.VmStatusDB[i].pod_status, vm_status))
                                self.send_notification(vm_name, vm_status, classifier, vm_namespace)
                                self.VmStatusDB[i].pod_status = vm_status

            # clear database - delete stale entries
            for entry in self.PodStatusDB:
                pod_name_namespace=entry.pod_name+":"+entry.pod_namespace
                if pod_name_namespace not in pods:
                    collectd.info("%s: %s %s/%s" % (PLUGIN_NAME, "Removing pod entry:", entry.pod_name, entry.pod_namespace))
                    classifier="pod:"+entry.pod_namespace+":"+entry.pod_name
                    self.send_pod_delete_notification("Pod", entry.pod_name, classifier, entry.pod_namespace)
                    #self.PodStatusDB.remove(entry)

                    # send replicaset delete pod notification
                    for rs in self.ReplicasetDB:
                        if rs.name in entry.pod_name:
                            rs_classifier="rs:"+rs.namespace+":"+rs.name
                            self.send_rs_pod_notification(rs_classifier, "deleted", rs.name, entry.pod_name)
                            rs.replicas=rs.replicas-1
                            # send deployment scaling down replicaset notification
                            for depl in self.DeploymentDB:
                                if depl.name in rs.name:
                                    depl_classifier="deployment:"+depl.namespace+":"+depl.name
                                    self.send_depl_scaling_notification(depl.name, "down", depl_classifier, rs.name, rs.replicas)

                    # send statefulset create pod notification
                    for ss in self.StatefulsetDB:
                        if ss.name in entry.pod_name:
                            ss_classifier="statefulset:"+ss.namespace+":"+ss.name
                            self.send_ss_pod_notification(ss_classifier, "deleted", ss.name, entry.pod_name)

                    self.PodStatusDB.remove(entry)
    
            for entry in self.VmStatusDB:
                vm_name_namespace=entry.pod_name+":"+entry.pod_namespace
                if vm_name_namespace not in vms:
                    collectd.info("%s: %s %s/%s" % (PLUGIN_NAME, "Removing vm entry:", entry.pod_name, entry.pod_namespace))
                    classifier="vm:"+entry.pod_namespace+":"+entry.pod_name
                    self.send_pod_delete_notification("Vm", entry.pod_name, classifier, entry.pod_namespace)
                    self.VmStatusDB.remove(entry)


        except Exception as e:
            collectd.error("k8s_notification plugin: read function failed: {}".format(e))


        try:
            cmd = "kubectl get service -A"
            all_services = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; %s" %(cmd), shell=True)).decode('utf8', errors='ignore')

            services = set()
            if all_services != "":
                for line in all_services.split('\n')[1:]:
                    if line != "":
                        svc_name = line.split()[1]
                        #log(svc_name)
                        svc_namespace = line.split()[0]
                        services.add(svc_name)

                        found = False
                        for i in range(len(self.SvcDB)):
                            if self.SvcDB[i].name == svc_name:
                                found = True
                                break

                        if not found:
                            collectd.info("%s: Adding entry for service: %s " % (PLUGIN_NAME, svc_name))
                            self.SvcDB.append(status_entry(svc_name, svc_namespace, current_time))
                            classifier="service:"+svc_namespace+":"+svc_name
                            self.send_create_notification("Service", "created", svc_name, classifier, svc_namespace)

            for entry in self.SvcDB:
                if entry.name not in services:
                    collectd.info("%s: %s %s" % (PLUGIN_NAME, "Removing service entry:", entry.name))
                    classifier="service:"+entry.namespace+":"+entry.name
                    self.send_create_notification("Service", "deleted", entry.name, classifier, entry.namespace)
                    self.SvcDB.remove(entry)

        except Exception as e:
            collectd.error("k8s_notification plugin: read function failed - {}".format(e))


        try:
            cmd = "kubectl get pv"
            all_pvs = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; %s" %(cmd), shell=True)).decode('utf8', errors='ignore')

            pvs = set()
            if all_pvs != "":
                for line in all_pvs.split('\n')[1:]:
                    if line != "":
                        pv_name = line.split()[0]
                        #log(pv_name)
                        pv_size = line.split()[1]
                        pv_status = line.split()[4]
                        pvs.add(pv_name)                   

                        found = False
                        for i in range(len(self.PvDB)):
                            if self.PvDB[i].name == pv_name:
                                found = True
                                break

                        classifier="pv:"+":"+pv_name
                        if not found:
                            collectd.info("%s: Adding entry for PV: %s capacity: %s, status: %s" % (PLUGIN_NAME, pv_name, pv_size, pv_status))
                            self.PvDB.append(pv_status_entry(pv_name, pv_size, pv_status, current_time))
                            self.send_pv_create_notification(pv_name, "Created", classifier, pv_size, pv_status)
                        else:
                            if self.PvDB[i].status != pv_status:
                                collectd.info("%s: PV %s status changed: %s -> %s" % (PLUGIN_NAME, pv_name, self.PvDB[i].status, pv_status))
                                self.send_pv_update_notification(pv_name, classifier, None, pv_status)
                                self.PvDB[i].status = pv_status
                                
                            if self.PvDB[i].size != pv_size:
                                collectd.info("%s: PV %s capacity changed: %s -> %s" % (PLUGIN_NAME, pv_name, self.PvDB[i].size, pv_size))
                                self.send_pv_update_notification(pv_name, classifier, pv_size, None)
                                self.PvDB[i].size = pv_size

            for entry in self.PvDB:
                if entry.name not in pvs:
                    collectd.info("%s: %s %s" % (PLUGIN_NAME, "Removing pv entry:", entry.name))
                    classifier="pv:"+":"+entry.name
                    self.send_pv_create_notification(entry.name, "Deleted", classifier, entry.size)
                    self.PvDB.remove(entry)

        except Exception as e:
            collectd.error("k8s_notification plugin: read function failed - {}".format(e))


def log(param):
    """
    Log messages to either collectd or stdout depending on how it was called.

    :param param: the message
    :return:  None
    """

    if __name__ != '__main__':
        collectd.info("%s: %s" % (PLUGIN_NAME, param))
    else:
        sys.stderr.write("%s\n" % param)
   
if __name__ != "__main__":
    # when running inside plugin register each callback
    inst = PodStatus()
    #collectd.register_init(inst.init)
    #collectd.register_config(inst.configure_callback)
    collectd.register_read(inst.read_function)

